/*
 *	array.h
 *
 *	This file exports the array class which is used to store
 *	any kind of objects.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_ARRAY_H_
#define	_OPaC_ARRAY_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

/****************************************************************/

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

/****************************************************************/

/*
 *	OPaC's Array class.
 */

class OP_PUBLIC OPArray : public OPContainer
{
	
protected:
	OPObjectRef*		references;				//	array of references
	
protected:
	OPArray ();
	virtual ~ OPArray ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
	virtual void InternalGetGapSize (Count pos, CountOut gap, CountOut num);
	virtual void InternalSetGapSize (Count pos, Count gap, Count num);
	
public:
	virtual void Delete ();									/* OVERRIDDEN */
	virtual OPIteratorRef NewIterator ();					/* OVERRIDDEN */
	
	virtual Bool Insert (OPObjectIn data);					/* OVERRIDDEN */
	virtual Bool Remove (OPObjectIn data, Count nth = 0);	/* OVERRIDDEN */
	virtual Bool Remove (Count index);
	virtual void RemoveAll ();
	
	virtual Bool SpecifyExtraction (OPArrayIn x);			/* OVERRIDDEN */
	
	virtual Bool Get (Count index, OPObjectOut object);		/* OVERRIDDEN */
	virtual Bool GetSimilar (OPObjectIn model, OPObjectOut object, Count nth = 0);	/* OVERRIDDEN */
	virtual int  CompareWith (OPObjectIn value) const;		/* OVERRIDDEN */
	
	virtual Bool LookFor (OPObjectIn model);				/* OVERRIDDEN */
	virtual Bool Find (OPObjectIn model, CountInOut pos);

	virtual Bool SetSize (Count size);
	virtual Bool SetMemZone (OPMemZone* zone);
	
	virtual OPObjectRef& Access (Count pos) const;
	virtual Bool Insert (Count pos, Count num = 1);
	virtual Bool Remove (Count pos, Count num);

	OP_DEFINE_CLASS (OPArray, OPContainer);
};

/****************************************************************/

/*
 *	If you want to allocate an array of a particular type of
 *	objects, just use "OP_ARRAY (Name)". This does an ugly
 *	type cast, but never mind, since the types are 100%
 *	compatible.
 */

#define OP_ARRAY(T)	(((T##Array*)(&(OP_NEW (OPArray))))[0])

/****************************************************************/

#endif /* _OPaC_ARRAY_H_ */

