/*
 *	internet.h
 *
 *	This file defines the classes related to the Internet (such
 *	as URL data downloading, etc.)
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_INTERNET_H_
#define	_OPaC_INTERNET_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_STRING_H_
#include <OPaC/string.h>
#endif

/****************************************************************/

OP_EXPORT_CLASS (OPInternet);					//	=> OPStorage

class OPaC_UrlHandle;

/****************************************************************/

class OP_PUBLIC OPInternet : public OPStream
{
	friend class OPaC_UrlHandle;
	
protected:
	OPaC_UrlHandle*		handle;					//	connection handle (if any)
	OPStringRef			agent;					//	agent's name (like for a browser)
	
	Bool				is_active;				//	TRUE => connection is open
	Bool				is_max_known;			//	TRUE => stream "length" is known
	
	Card64				data_read;				//	current data read (including buffer)
	Card64				data_offset;			//	current data offset
	Card64				data_max;				//	maximum # of bytes (if known)
	
protected:
	OPInternet ();
	~OPInternet ();

	//	Method calls reserved for internal notifications, as sent by the private
	//	and OS dependent class OPaC_UrlHandle.
	
	virtual void NotifyNewData (Count num);
	virtual void NotifyMaximum (Card64 num);
	virtual void NotifyError ();
	
public:
	virtual void Delete ();						/* OVERRIDDEN */
	
	virtual Bool OpenUrl (OPStringIn url);
	virtual Bool OpenUrl (OPStringIn url, OPStringIn headers);
	virtual Bool CloseUrl ();
	
	virtual void SetAgentName (OPStringIn name);
	virtual Bool GetAgentName (OPStringOut name) const;
	
	virtual Bool GetReadyBytes (CountOut num) const;
	virtual Bool GetMaxBytes (Card64Out num) const;
	
	virtual Bool IsConnectionAlive () const;
	
	//	Inherited, general methods used to read data streams:
	
	virtual Count Read (void*, Count);			/* OVERRIDDEN */
	virtual Count Skip (Count);					/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPInternet, OPStream);
};

/****************************************************************/

/*
 *	Internal, low level URL handler class.
 */

class OPaC_UrlHandle
{
	void*				internal;
	OPInternet*			internet;

private:
	void Close ();

public:
	OPaC_UrlHandle ();
	~ OPaC_UrlHandle ();
	
	Bool Connect (OPInternet*, const LatinChar*, const LatinChar*, const LatinChar*, Count);
	Bool Read (void*, Count&, Bool wait = TRUE);
	Bool Skip (Count&, Bool wait = TRUE);
};

/****************************************************************/

#endif /* _OPaC_INTERNET_H_ */
