/*
 *	monitorvalue.h
 *
 *	Monitor one (or several) value(s) and display them in a
 *	view (e.g. a gauge, a scope, etc.)
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_MONITORVALUE_H_
#define	_OPaC_MONITORVALUE_H_

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_VALUES_H_
#include <OPaC/values.h>
#endif

#ifndef _OPaC_XTSMARTSRC_H_
#include <OPaC/xtsmartsrc.h>
#endif

/****************************************************************/


class OP_PUBLIC OPMonitorValue : public OPControl, public OPxSmartSource
{
	
protected:
	mutable OPValueRef	value;				//	internal value
	
protected:
	OPMonitorValue ();
	virtual ~ OPMonitorValue ();
	
	virtual void StoreInstance (OPStorageIn storage);
	virtual void RestoreInstance (OPStorageIn storage);
	
public:
	virtual Bool SetValue (OPValueIn value);
	virtual OPValueRef ReturnValue () const;
	
	virtual void FlushCacheAndRedraw ();
	
	//	Overridden public methods :
	
	virtual void Delete ();
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);
	virtual void Broadcast (OPAtomIn message, OPObjectIn value);
	
	OP_DEFINE_SLOT (OPValue, value, SetValue, ReturnValue);
	
	OP_DEFINE_CLASS (OPMonitorValue, OPControl);
};

/****************************************************************/

#endif /* _OPaC_MONITORVALUE_H_ */

