/*
 *	objectwell.h
 *
 *	The Object Well class implements a drop target for objects...
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_OBJECTWELL_H_
#define	_OPaC_OBJECTWELL_H_

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

/****************************************************************/

/*
 *	The Object Well class provides a drag & drop target for
 *	objects and optionally provides an easy way to edit the
 *	simpler ones (usually text fields).
 */

class OP_PUBLIC OPObjectWell : public OPControl
{

protected:
	OPObjectRef			object;					//	object represented in the well
	OPHashTableRef		reporters;				//	reporters created for the edition
	OPTypeInfoRef		base_type;				//	restricting base type
	
	Bool				has_edit_support;		//	TRUE => support in-place edition
	Bool				has_right_arrow;		//	TRUE => draw the right arrow
	Bool				has_direct_editor;		//	TRUE => a direct editor has been set up
	
	Bool				use_new_button;			//	TRUE => use new button
	Bool				use_kill_button;		//	TRUE => use kill button
	
	OPLookIcon			icon_cube;				//	usually OP_ICON_CUBE_NORMAL
	OPLookIcon			icon_arrow;				//	usually OP_ICON_RIGHT_ARROW
	
	OPCoord				editor_width;			//	width of editor object
	OPViewRef			editor;					//	externally provided editor view
	OPIconRef			kill_icon;				//	kill icon for button
	OPButtonRef			kill_button;			//	kill button => remove the object
	OPIconRef			new_icon;				//	new icon for button
	OPButtonRef			new_button;				//	new button => replace the object
	
	OPCoord				well_arrow_dx;			//	width of the large arrow
	OPCoord				well_dx;				//	width of the drop well
	OPCoord				well_dy;				//	height of the drop well
	OPCoord				well_border;			//	margin around the well
	OPCoord				well_cube_oy;			//	vertical offset for cube
	OPCoord				well_arrow_oy;			//	vertical offset for arrow
	
protected:
	OPObjectWell ();
	virtual ~ OPObjectWell ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void CleanupWell ();
	virtual void SetupWellForObject ();
	virtual void AdjustObject (OPObjectIn object);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetObject (OPObjectIn object);
	virtual Bool SetObjectEditor (OPViewIn object);
	virtual Bool SetBaseType (OPTypeInfoIn type);
	
	virtual Bool GetObject (OPObjectOut object) const;
	virtual Bool GetBaseType (OPTypeInfoOut type) const;
	
	virtual OPObjectRet ReturnObject () const;
	virtual OPViewRet ReturnObjectEditor () const;
	
	virtual Bool SetNewButton (Bool yes);
	virtual Bool SetKillButton (Bool yes);
	virtual Bool SetWellIcons (OPLookIcon cube, OPLookIcon arrow);
	
	virtual Bool Update ();
	virtual Bool NewObject ();
	virtual Bool KillObject ();
	virtual void AddReporter (OPDataReportIn reporter);
	
	virtual void GetMinSize (OPCoordOut, OPCoordOut) const;				/* OVERRIDDEN */
	virtual Bool DragAndDrop (OPRectIn, OPObjectIn, OPDragDropProtocol);/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp);	/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (OPObject, object, SetObject, ReturnObject);
	OP_DEFINE_SLOT (OPView, editor, SetObjectEditor, ReturnObjectEditor);
	OP_DEFINE_CLASS (OPObjectWell, OPControl);
};

/****************************************************************/

#endif /* _OPaC_OBJECTWELL_H_ */
