/*
 *	phantom.h
 *
 *	A phantom object just overrides the "Resolve" and "CheckedResolve"
 *	methods in order to make the references think they refer to some
 *	other, real, object. The phantom will never be seen !
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_PHANTOM_H_
#define	_OPaC_PHANTOM_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

/****************************************************************/

class OP_PUBLIC OPPhantom : public OPObject
{
	
private:
	OPObjectRef			root;				//	root object
	OPStringRef			path;				//	path for the real object
	OPObjectRef			cache;				//	cached object
	
	Bool				is_cache_valid;		//	TRUE => the cache contains valid data
	Bool				is_const_object;	//	TRUE => just query the root once
	
protected:
	OPPhantom ();
	virtual ~ OPPhantom ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void UpdateCache ();
	
public:
	virtual Bool Initialise (OPObjectIn root, OPStringIn path, Bool is_const_object);
	
	OPObjectRet ReturnRoot () const			{ return this->root; }
	OPStringRet ReturnPath () const			{ return this->path; }
	Bool IsConstObject () const				{ return this->is_const_object; }
	
	virtual OPObject* Resolve ();						/* OVERRIDDEN */
	virtual OPObject* CheckAndResolve ();				/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPPhantom, OPObject);
};

/****************************************************************/

#endif /* _OPaC_PHANTOM_H_ */
