/*
 *	file_io.h
 *
 *	The File I/O functions support low-level access to the files. These should
 *	in no case be used directly by the developer !
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SD_FILE_IO_H_
#define	_OPaC_SD_FILE_IO_H_

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif

/****************************************************************/

typedef Int64			OPaC_FilePos;

/****************************************************************/

/*
 *	OPaC low level file access functions :
 */

class OP_PUBLIC OPaC_File
{
public:
	enum Mode
	{
		OPEN_NONE			= 0x0000,		//	not open
		OPEN_READ			= 0x0001,		//	open with read OK
		OPEN_WRITE			= 0x0002,		//	open with write OK
		
		OPEN_APPEND			= 0x0004,		//	OPEN_WRITE: append requested
		OPEN_TRUNC			= 0x0008,		//	OPEN_WRITE: truncate when opening
		OPEN_CREATE			= 0x0010,		//	OPEN_WRITE: create file if needed
		OPEN_SYNC			= 0x0020,		//	OPEN_WRITE: synchronise writes
		
		OPEN_COPY_ON_WRITE	= 0x0100		//	for mapped memory: copy-on-write
	};
	
private:
	void*				desc;
	void*				share_base;
	unsigned long		share_size;
	void*				share_desc;
	Mode				mode;
	
public:
	OPaC_File ();
	~OPaC_File ();
	
	Bool Open (OPStringIn name, Mode mode);
	Bool Close ();
	
	Size Read (void* buffer, Size size);
	Size Write (const void* buffer, Size size);
	
	void* MapSharedMemory (OPaC_FilePos offset, Size size, OPStringIn map_name = OPStringNull);
	Bool  UnmapSharedMemory ();
	Bool  FlushSharedMemory (void* base, Size size);

	Bool SeekRel (OPaC_FilePos offset);
	Bool SeekAbs (OPaC_FilePos offset);
	OPaC_FilePos TellPos () const;
	OPaC_FilePos TellSize () const;

	Bool Truncate ();
	Bool Resize (OPaC_FilePos size);
	Bool Flush ();

	static Bool DeleteFile (OPStringIn name);
	
	static Bool CreateDirectory (OPStringIn name);
	static Bool DeleteDirectory (OPStringIn name);

	static Bool SetCurrentDirectory (OPStringIn name);
	static Bool GetCurrentDirectory (OPStringOut name);
	static Bool GetOPaCDirectory (OPStringOut name);
	static Bool GetInitialDirectory (OPStringOut name);
	
	static Bool ParsePathAndReplace (const char* path, char* buffer, int size);
};

/****************************************************************/

#endif /* _OPaC_SD_FILE_IO_H_ */

