/*
 *	shadeview.h
 *
 *	The shade view draws a rectangular shade behind a specified
 *	view.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SHADEVIEW_H_
#define	_OPaC_SHADEVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

/*
 *	The shade view draws a translucent shade behind a view.
 */

class OP_PUBLIC OPShadeView : public OPView
{
	
protected:
	OPViewRef			view;				//	draw a shade behind this view
	OPColorPixel		color_ext;			//	color of shade (external)
	OPColorPixel		color_int;			//	color of shade (internal)
	Int32				dx_offset;			//	horizontal offset
	Int32				dy_offset;			//	vertical offset
	Int32				grow;				//	positive => larger than view
	Card32				transition;			//	size of transition area color 1 => 2.
	
protected:
	OPShadeView ();
	virtual ~ OPShadeView ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetView (OPViewIn view);
	virtual Bool SetColorExt (OPColorPixelIn color);
	virtual Bool SetColorInt (OPColorPixelIn color);
	virtual Bool SetOffset (Int32In dx, Int32 dy);
	virtual Bool SetDxOffset (Int32In offset);
	virtual Bool SetDyOffset (Int32In offset);
	virtual Bool SetGrow (Int32In grow);
	virtual Bool SetTransition (Card32In transition);
	
	virtual OPViewRet ReturnView () const;
	virtual OPColorPixel ReturnColorExt () const;
	virtual OPColorPixel ReturnColorInt () const;
	virtual Int32 ReturnDxOffset () const;
	virtual Int32 ReturnDyOffset () const;
	virtual Int32 ReturnGrow () const;
	virtual Card32 ReturnTransition () const;
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (OPColorPixel, color_ext, SetColorExt, ReturnColorExt);
	OP_DEFINE_SLOT (OPColorPixel, color_int, SetColorInt, ReturnColorInt);
	OP_DEFINE_SLOT (Int32, dx_offset, SetDxOffset, ReturnDxOffset);
	OP_DEFINE_SLOT (Int32, dy_offset, SetDyOffset, ReturnDyOffset);
	OP_DEFINE_SLOT (Int32, grow, SetGrow, ReturnGrow);
	OP_DEFINE_SLOT (Card32, transition, SetTransition, ReturnTransition);
	
	OP_DEFINE_CLASS (OPShadeView, OPView);
};

/****************************************************************/

#endif /* _OPaC_SHADEVIEW_H_ */
