/*
 *	slider.h
 *
 *	Slider class definition.
 *	
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SLIDER_H_
#define	_OPaC_SLIDER_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_SCROLLER_H_
#include <OPaC/scroller.h>
#endif

/****************************************************************/

/*
 *	Slider class.
 */

class OP_PUBLIC OPSlider : public OPScroller
{
	
protected:
	Real				slider_scale;			//	slider scale (multiplication factor for value)
	Real				slider_offset;			//	slider offset (pos. of zero in slider)
	
protected:
	OPSlider ();
	virtual ~ OPSlider ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn);				/* OVERRIDDEN */
	
	virtual void InitArrow (OPIconInOut arrow, Card32);	/* OVERRIDDEN */
	virtual void InitIcons (Bool);						/* OVERRIDDEN */
	virtual void UpdateKnobSize ();						/* OVERRIDDEN */
	virtual void CheckSpaceForParts ();					/* OVERRIDDEN */
	
	virtual void DrawKnob (OPGrailPortIn port);			/* OVERRIDDEN */
	virtual void DrawBody (OPGrailPortIn port);			/* OVERRIDDEN */
	virtual void Scroll (Card32 id);					/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void SetArrowPos (OPScrollArrowPos pos);	/* OVERRIDDEN */
	
	virtual Bool SetSliderValue (Real value);
	virtual Bool SetSliderScale (Real scale);
	virtual Bool SetSliderOffset (Real offset);
	
	virtual Real ReturnSliderValue () const;
	virtual Real ReturnSliderScale () const;
	virtual Real ReturnSliderOffset () const;
	
	OP_DEFINE_SLOT (Real, slider_value,  SetSliderValue,  ReturnSliderValue);
	OP_DEFINE_SLOT (Real, slider_scale,  SetSliderScale,  ReturnSliderScale);
	OP_DEFINE_SLOT (Real, slider_offset, SetSliderOffset, ReturnSliderOffset);
	OP_DEFINE_CLASS (OPSlider, OPScroller);
};

/****************************************************************/

#endif /* _OPaC_SLIDER_H_ */
