/*
 *	sys.h
 *
 *	This file contains system dependent type and constant definitions. The
 *	higher level software should rely on these information rather than using
 *	system related header files.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SYS_H_
#define	_OPaC_SYS_H_

#ifndef _OPaC_COMPILER_H_
#include <OPaC/compiler.h>
#endif


/*
 *	Machine dependent types.
 */

OP_TYPEDEF (Card64, Address);					//	generalised 64-bit address
OP_TYPEDEF (Card32, Size);						//	memory size specifier


/*
 *	Miscellanous machine dependent macros, constants and
 *	symbols.
 */

#define	SYS_SIZE(x)		((Size)(x))
#define	SYS_ADDRESS(x)	((Address)(x))
#define	SYS_ALIGN(x)	((SYS_ADDRESS(x) + SYS_ALIGN_ROUND) & SYS_ALIGN_MASK)

#define	SYS_LITTLE_ENDIAN						//	little endian Intel processor
#undef	SYS_BIG_ENDIAN							//	not a Motorola (big => great => Motorola)

#define	SYS_PTR_SIZE	SYS_SIZE (sizeof (Address))	//	pointer size in bytes
#define	SYS_MAX_SIZE	SYS_SIZE (0xFFFFFFFF)		//	maximum 32-bit size

#define	SYS_ALIGN_SIZE	SYS_ADDRESS (16)
#define	SYS_ALIGN_ROUND	SYS_ADDRESS (SYS_ALIGN_SIZE-1)
#define	SYS_ALIGN_MASK	SYS_ADDRESS (~SYS_ALIGN_ROUND)
#define	SYS_ALIGN_SHIFT	(4)

#define	SYS_CAST(T,x)	(((T*)(& x))[0])


/*
 *	Endian-dependent data access macros. These are used by
 *	the input/output functions, since the data is always
 *	stored in big endian representation.
 */

#ifdef	SYS_BIG_ENDIAN
#define	SYS_DATA_8(x)	(x)
#define	SYS_DATA_16(x)	(x)
#define	SYS_DATA_32(x)	(x)
#endif

#ifdef	SYS_LITTLE_ENDIAN
#define	SYS_DATA_8(x)	(x)
#define	SYS_DATA_16(x)	(swap_16_bit ((Card16)(x)))
#define	SYS_DATA_32(x)	(swap_32_bit ((Card32)(x)))
#endif



/*
 *	Byte swapping functions used to convert data to and from
 *	the input/output stream.
 */

extern inline Card16
swap_16_bit (Card16 x)
{
	return (Card16)((x >> 8) | (x << 8));
}

extern inline Card32
swap_32_bit (Card32 x)
{
	return (Card32)( (x << 24)						//	00'00'00'xx => xx'00'00'00
				   | ((x & 0x0000FF00) << 8)		//	00'00'xx'00 => 00'xx'00'00
				   | ((x >> 8) & 0x0000FF00)		//	00'xx'00'00 => 00'00'xx'00
				   | (x >> 24) );					//	xx'00'00'00 => 00'00'00'xx
}

#endif
