/*
 *	typeinfo.h
 *
 *	This file defines the type information class used to collect
 *	and represent the OPaC types defined within a project.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_TYPEINFO_H_
#define	_OPaC_TYPEINFO_H_

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif

/****************************************************************/

/*
 *	The class provides encapsulated type information.
 */

class OP_PUBLIC OPTypeInfo : public OPObject
{

protected:
	OPTypeDef			type;				//	data type description
	OPBitsetInfoRef		bitset;				//	bitset information associated (if any)
	Bool				is_read_ok;			//	TRUE => can be read from
	Bool				is_write_ok;		//	TRUE => can be written to
	
protected:
	OPTypeInfo ();
	virtual ~ OPTypeInfo ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Initialise (OPTypeDef type, OPBitsetInfoIn info = OPBitsetInfoNull, Bool rd = TRUE, Bool wr = TRUE);
	virtual void Initialise (OPRuntimeIn runtime);
	
	virtual Bool GetRuntimeInfo (OPRuntimeOut runtime) const;
	
	OPTypeDef ReturnType () const						{ return this->type; }
	const OPBitsetInfoRef& ReturnBitsetInfo () const	{ return this->bitset; }
	Bool ReturnReadOk () const							{ return this->is_read_ok; }
	Bool ReturnWriteOk () const							{ return this->is_write_ok; }
	
	//	Global methods for global types :
	
	static Count ReturnNumTypes ();
	static OPTypeDef ReturnNthType (Count index);
	static const OPAtomRef& ReturnNthName (Count index);
	
	static Bool FindType (const LatinChar* name, OPTypeDefOut type);
	static Bool FindType (OPAtomIn name, OPTypeDefOut type);
	
	OP_DEFINE_CLASS (OPTypeInfo, OPObject);
};

/****************************************************************/

#endif /* _OPaC_TYPEINFO_H_ */
