/*
 *	filedir.cxx
 *
 *	The File Dir class provides access to directories.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"

#include <OPaC/container.h>
#include <OPaC/new.h>
#include <OPaC/string.h>

#include <OPaC/sd/filedir.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;

/****************************************************************/

#define MAX_PATH 256

class OPaC_PlatformFileDir : public OPaC_FileDir
{
private:
	Bool				is_first;
	
protected:
	virtual ~ OPaC_PlatformFileDir ();
	
public:
	OPaC_PlatformFileDir ();
	
	virtual Bool Open (OPStringIn path, OPStringIn pattern);
	virtual Bool Close ();
	virtual Bool FindNext (OPaC_FileInfoOut info);
	virtual Bool FindNext (OPaC_FileInfoOut info, OPDictOut attr);
};

/****************************************************************/

OPaC_FileDir*
OPaC_FileDir::New ()
{
	return NEW (OPaC_PlatformFileDir);
}



OPaC_PlatformFileDir::OPaC_PlatformFileDir ()
{
	this->is_first = FALSE;
}

OPaC_PlatformFileDir::~OPaC_PlatformFileDir ()
{
	if (this->is_open) {
		this->Close ();
	}
}

Bool
OPaC_PlatformFileDir::Open (OPStringIn path, OPStringIn pattern)
{
	char buf_path[MAX_PATH+MAX_PATH];
	char buf_pattern[MAX_PATH];
	
	strcpy (buf_path, ".");
	strcpy (buf_pattern, "*");
	
	if ( (path.IsValid ())
	  && (path->CountCharacters ()) ) {
		path->GetStringCopy (buf_path, MAX_PATH);
	}
	if ( (pattern.IsValid ())
	  && (pattern->CountCharacters ()) )  {
		pattern->GetStringCopy (buf_pattern, MAX_PATH);
	}
	
	char* name = buf_path;
	
	while (*name) {
		if (*name == '/') *name = '\\';
		name++;
	}
	
	if (name[-1] != '\\') {
		name[0] = '\\';
		name[1] = 0;
	}
	
	name = buf_path;
	strcat (name, buf_pattern);
	
	this->Close ();
	
	this->is_first = TRUE;
	this->is_open  = TRUE;
	
	return TRUE;
}

Bool
OPaC_PlatformFileDir::Close ()
{
	this->is_open = FALSE;
	
	return TRUE;
}

Bool
OPaC_PlatformFileDir::FindNext (OPaC_FileInfoOut info)
{
	if (! this->is_first) {
			return FALSE;
	} else {
		this->is_first = FALSE;
	}
	
	info.name = "dummy";
	info.size = (Card64)12345;
	
	info.is_directory = FALSE;
	info.is_link      = FALSE;
	info.is_hidden    = FALSE;
	info.is_read_ok   = TRUE;
	info.is_write_ok  = TRUE;
	
	return TRUE;
}

Bool
OPaC_PlatformFileDir::FindNext (OPaC_FileInfoOut info, OPDictOut attr)
{
	if (this->FindNext (info)) {
		attr = OPDictNull;
		return TRUE;
	}
	
	return FALSE;
}
