/*
 *	sd_grailwdo.cxx
 *
 *	GRAphical Interface Layer for building graphical interface
 *	elements. This is highly machine dependent ! Implementation for X11.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

// Modified by Laurent Bovet 08.02.98

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"
#include "../common/sd_grafix.h"

#include <stdio.h>

#include <OPaC/grail.h>
#include <OPaC/new.h>
#include <OPaC/window.h>

/****************************************************************/

Count	OPaC_GrWindow::counter = 0;
char    OPaC_GrWindow::globname[64] = { 0 };

/****************************************************************/

#define	MAX_TITLE_LEN	200

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPGrailWindow);

/****************************************************************/

/*
 *	Construct a new window instance. This will build an
 *	independent window class; you might think this is
 *	expensive, but it is much easier to implement that
 *	way !
 */

OPGrailWindow::OPGrailWindow ()
{
	OP_METHOD (OPGrailWindow, 0);
	
	OPaC_GrWindow* window = OP_ALLOC_FROM_ZONE (this->ReturnMemZone (), OPaC_GrWindow);
	
	this->bounding_box.MakeEmpty ();
	this->window = window;
	this->view   = 0;
	this->style  = OP_WIN_NAKED;
	
	this->is_active  = FALSE;
	this->is_open    = FALSE;
	this->is_reduced = FALSE;
	
	// Create the window name.
	
	if (OPaC_GrWindow::globname[0] == 0) {
		snprintf (OPaC_GrWindow::globname, sizeof (OPaC_GrWindow::globname),
				  GLOBNAME_ID "%d",				//	"OPaCgrail:" + 1 id
				  (unsigned int) getpid());
	}

	sprintf (window->wclass_name, "%s-%d", OPaC_GrWindow::globname,
										   OPaC_GrWindow::counter++);
	
	window->xwindow      = 0;
}



/*
 *	Destroy a window : this will delete the window class
 *	and the window instance.
 */

OPGrailWindow::~OPGrailWindow ()
{
	OP_METHOD (~OPGrailWindow, 0);
	
	OPaC_GrWindow* window = this->window;

	XEvent xevent;

	if (window) {
		
		if (window->xwindow) {

			XDestroyWindow (xdpy, window->xwindow);
			
			// Remove all events about the window in X event queue
			
			xevent.type = 0;
			
			while (xevent.type != DestroyNotify) {
				
			    XWindowEvent (xdpy, window->xwindow,
							  SubstructureNotifyMask |
							  StructureNotifyMask |
							  ExposureMask |
							  KeyPressMask | KeyReleaseMask |
							  ButtonPressMask | ButtonReleaseMask |
							  FocusChangeMask |
							  PointerMotionMask, &xevent);
			}
		}
		OP_FREE_FROM_ZONE (this->ReturnMemZone (), window);
	}
}



void
OPGrailWindow::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	OP_ABORT ("Trying to save Grail Window");
}



void
OPGrailWindow::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	OP_ABORT ("Trying to restore Grail Window");
}



/*
 *	Create a window with a title bar and a minimum adorn-
 *	ment, depending on the specified style.
 *
 *	NB: The top left corner of the window will be [x1, y1]
 *	    and the client size (the part you can draw into)
 *		is specified by x2-x1+1 and y2-y1+1.
 */

void
OPGrailWindow::Initialise (OPWindowIn view,
						   OPWindowStyle style,
						   OPStringIn name,
						   OPRectIn box, OPRelCoord margin)
{
	OP_METHOD (Initialise, 5);
	OP_ARG (1, OPWindowIn, view);
	OP_ARG (2, OPWindowStyle, style);
	OP_ARG (3, OPStringIn, name);
	OP_ARG (4, OPRectIn, box);
	OP_ARG (5, OPRelCoord, margin);
	OP_ASSERT ((OPRelCoord)(box.DX ()) > (2*margin));
	OP_ASSERT ((OPRelCoord)(box.DY ()) > (2*margin));
	
	OPCoord x1 = box.X1 () + margin;
	OPCoord y1 = box.Y1 () + margin;
	OPCoord x2 = box.X2 () - margin;
	OPCoord y2 = box.Y2 () - margin;
	
	this->Initialise (view, style, name, x1, y1, x2, y2);
}

void
OPGrailWindow::Initialise (OPWindowIn view,
						   OPWindowStyle style, OPStringIn name,
						   OPCoord x1, OPCoord y1, OPCoord x2, OPCoord y2)
{
	OP_METHOD (Initialise, 7);
	OP_ARG (1, OPWindowIn, view);
	OP_ARG (2, OPWindowStyle, style);
	OP_ARG (3, OPStringIn, name);
	OP_ARG (4, OPCoord, x1);
	OP_ARG (5, OPCoord, y1);
	OP_ARG (6, OPCoord, x2);
	OP_ARG (7, OPCoord, y2);
	OP_ASSERT ((x2 > x1) && (y2 > y1));
	
	Window xwindow;
	XSetWindowAttributes xattributes;
	
	//	Translate the window mode :
	
	switch (style & OP_WIN_MODE_MASK) {
	    
		case OP_WIN_NAKED:
			xattributes.override_redirect = 1;
			break;
		
		case OP_WIN_NORMAL:
			xattributes.override_redirect = 0;
			break;
		
		case OP_WIN_TITLE:
			xattributes.override_redirect = 0;
			break;
	}
	
	XSizeHints xsizehints;
  
	xsizehints.flags  = USPosition | USSize;
	xsizehints.x      = (int)x1;
	xsizehints.y      = (int)y1;
	xsizehints.width  = (int)(x2-x1+1);
	xsizehints.height = (int)(y2-y1+1);
	
	//	Translate the size window attributes :
	
	if (!(style & OP_WIN_SIZE_OK)) {
	    xsizehints.flags |= (PMinSize | PMaxSize);
		xsizehints.min_width = (int)(x2-x1+1);
		xsizehints.min_height = (int)(y2-y1+1);
		xsizehints.max_width = (int)(x2-x1+1);
		xsizehints.max_height = (int)(y2-y1+1); 
	}
	
	this->style = style;
	
	//	Now create the corresponding window. We have to extract the window name
	//	and convert it to ISO Latin-1 encoding.
	
	OPCoord xl = 0;
	OPCoord xr = 0;
	OPCoord yt = 0;
	OPCoord yb = 0;
	
	OPGrailWindow::GetFrameSize (style, xl, yt, xr, yb);
	
	LatinChar name_copy[MAX_TITLE_LEN];
	Count     length = name.IsValid () ? name->CountCharacters () : 0;
	
	name->GetStringCopy (name_copy, MAX_TITLE_LEN);

	// Create the window

	xwindow = XCreateSimpleWindow (xdpy,
								   RootWindow (xdpy, DefaultScreen(xdpy)),
								   (int)x1,
								   (int)y1,
								   (int)(x2-x1+1),
								   (int)(y2-y1+1),
								   0,
								   BlackPixel (xdpy, DefaultScreen (xdpy)),
								   BlackPixel (xdpy, DefaultScreen (xdpy)));
	
	OP_ASSERT (xwindow);
	
	//  Transmit the size hints
	
	XSetWMSizeHints (xdpy, xwindow, &xsizehints, XA_WM_NORMAL_HINTS);
	
	
	//  Set the window manager protocol in order to receive notification when
	//  the user or the window manager wants to destroy the window
	
	Atom xwm_delete_window = XInternAtom (xdpy, "WM_DELETE_WINDOW", False);
	
	XSetWMProtocols (xdpy, xwindow, &xwm_delete_window, 1);
	
	//  Optimize redraw under the window when moving.
	//  Efficient for "small" windows
	
	xattributes.save_under = ( (x2-x1)*(y2-y1) < 5000 ) ? True : False;
	
	//  Set the frame and optimized redraw attributes
	
	XChangeWindowAttributes (xdpy, xwindow, CWOverrideRedirect | CWSaveUnder,
							 &xattributes);
	
	//  Store the name
	
	XStoreName (xdpy, xwindow, name_copy);							 
	
	this->window->xwindow = xwindow;
	this->view = view;
	this->bounding_box = OPRect (x1, y1, x2, y2);
	
	//  Register the property names and set the properties for the
	//  current window
	
	char inst[32];
	Atom xinst_atom;
	Atom xid_atom;

	xinst_atom = XInternAtom (xdpy, OP_XPROP_WIN_INST, False);
	xid_atom   = XInternAtom (xdpy, OP_XPROP_WIN_ID, False);
	
	snprintf (inst, sizeof (inst), "%x", (unsigned long)(OPGrailWindow*)this);
	
	XChangeProperty (xdpy, this->window->xwindow,
					 xinst_atom,
					 XA_STRING,
					 8,
					 PropModeReplace,
					 inst,
					 strlen(inst));
	
	XChangeProperty (xdpy, this->window->xwindow,
					 xid_atom,
					 XA_STRING,
					 8,
					 PropModeReplace,
					 this->window->wclass_name,
					 strlen(this->window->wclass_name));
	
	//  Tell what events the window will accept
	
	Card32 mask = SubstructureNotifyMask
	            | StructureNotifyMask
	            | ExposureMask
	            | KeyPressMask
	            | KeyReleaseMask
	            | ButtonPressMask
	            | ButtonReleaseMask
	            | FocusChangeMask
	            | PointerMotionMask;
	
	XSelectInput (xdpy, this->window->xwindow, mask);
									
	super::Initialise (this, x2-x1 + 1, y2-y1 + 1);
}
	


/*
 *	Select another name for the current window. This is only
 *	possible if the window has been defined with a title bar.
 */

void
OPGrailWindow::ChangeName (OPStringIn name)
{
	OP_METHOD (ChangeName, 1);
	OP_ARG (1, OPStringIn, name);
	
	//	Convert the name to ISO Latin-1 encoding to keep Windows happy.
	
	LatinChar name_copy[200];
	Count     length = name.IsValid () ? name->CountCharacters () : 0;
	
	if (length) {
		if (length > (200-1)) length = 200-1;
		name->GetStringCopy (name_copy, length);
	}
	
	name_copy[length] = 0;
	XStoreName (xdpy, this->window->xwindow, name_copy);
}



void
OPGrailWindow::Open ()
{
	OP_METHOD (Open, 0);
	
	if ( (this->is_open == TRUE)
	  && (this->is_reduced == FALSE) )
		return;
	
	//  Tell the X server to display the window

	XMapWindow (xdpy, this->window->xwindow);
	
	this->is_open    = TRUE;
	this->is_reduced = FALSE;
}



void
OPGrailWindow::Reduce ()
{
	OP_METHOD (Reduce, 0);
	
	XIconifyWindow (xdpy, this->window->xwindow, DefaultScreen (xdpy));
	
	this->is_open    = TRUE;
	this->is_reduced = TRUE;
}



void
OPGrailWindow::Close ()
{
	OP_METHOD (Close, 0);
	
	if (this->is_open == FALSE)
		return;

	XUnmapWindow (xdpy, this->window->xwindow);	
	
	this->is_open    = FALSE;
	this->is_reduced = FALSE;
}


void
OPGrailWindow::ActivateAsFrontWindow (Bool front)
{
  OP_METHOD (ActivateAsFrontWindow, 1);
  OP_ARG (1, Bool, front);

  XSetInputFocus (xdpy, this->window->xwindow, RevertToNone, CurrentTime);
}


void
OPGrailWindow::SysChangedActive (Bool yes)
{
	OP_METHOD (SysChangedActive, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_active = yes;
	this->view->SysChangedActive (yes);
}

void
OPGrailWindow::SysChangedOpen (Bool yes)
{
	OP_METHOD (SysChangedOpen, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_open = yes;
	this->view->SysChangedOpen (yes);
}

void
OPGrailWindow::SysChangedVisible (Bool yes)
{
	OP_METHOD (SysChangedVisible, 1);
	OP_ARG (1, Bool, yes);
	
	this->view->SysChangedVisible (yes);
}
 
void
OPGrailWindow::SysChangedReduced (Bool yes)
{
	OP_METHOD (SysChangedReduced, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_reduced = yes;
	this->view->SysChangedReduced (yes);
}

	
Bool
OPGrailWindow::IsActive () const
{
	OP_METHOD (IsActive, 0);
	
	return this->is_active;
}


/*
 *	Set window's origin. This will be reflected immediately
 *	by the window manager so that this can be used for
 *	dragging.
 */

void
OPGrailWindow::SetOrigin (OPCoord x, OPCoord y)
{
	OP_METHOD (SetOrigin, 2);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	
	if ( (this->window)
	  && (this->window->xwindow) ) {
		
 		XMoveWindow (xdpy, this->window->xwindow,
					 (int)x,
					 (int)y);
	}
}



/*
 *	Return the window's origin.
 */

void
OPGrailWindow::GetOrigin (OPCoordOut x, OPCoordOut y) const
{
	OP_METHOD (GetOrigin, 2);
	OP_ARG (1, OPCoordOut, x);
	OP_ARG (2, OPCoordOut, y);
	
	x = this->bounding_box.X1 ();
	y = this->bounding_box.Y1 ();
}



/*
 *	Set the window's size.
 */

void
OPGrailWindow::SetSize (OPCoord dx, OPCoord dy)
{
	OP_METHOD (SetSize, 2);
	OP_ARG (1, OPCoord, dx);
	OP_ARG (2, OPCoord, dy);

	if ( (this->bounding_box.DX () == dx)
	  && (this->bounding_box.DY () == dy) ) {
		return;
	}
	
	if ( (this->window)
	  && (this->window->xwindow) ) {
		
		this->bounding_box.SetSize (dx, dy);
		
		XResizeWindow (xdpy,
					   this->window->xwindow,
					   (unsigned int) dx,
					   (unsigned int) dy);
		
		super::Initialise (this->depth,
						   this->bounding_box.DX (),
						   this->bounding_box.DY ());
	}
}


/*
 *	Return the window's size.
 */

void
OPGrailWindow::GetSize (OPCoordOut dx, OPCoordOut dy) const
{
	OP_METHOD (GetOrigin, 2);
	OP_ARG (1, OPCoordOut, dx);
	OP_ARG (2, OPCoordOut, dy);
	
	dx = this->bounding_box.DX ();
	dy = this->bounding_box.DY ();
}


/*
 *	The user either resized or moved the window, or both,
 *	so we have to take care of it.
 */

void
OPGrailWindow::UserChangedGeometry ()
{
	OP_METHOD (UserChangedGeometry, 0);
	
	Window xwindow, xrootwindow, child;
	unsigned int dx, dy, xborderwidth, xdepth;
	int x, y;	

	OPRect box = this->bounding_box;

	//  Get width and height

	XGetGeometry (xdpy, this->window->xwindow,
				  &xrootwindow, &x, &y, &dx, &dy, &xborderwidth, &xdepth);
	
	x = 0;
	y = 0;

	// Get position
	
	XTranslateCoordinates (xdpy, this->window->xwindow,
						   DefaultRootWindow (xdpy),
						   0, 0, &x, &y, &child);
	
 	this->bounding_box.X1 () = (OPCoord) x;
 	this->bounding_box.Y1 () = (OPCoord) y;
 	this->bounding_box.X2 () = (OPCoord) x + dx - 1;
 	this->bounding_box.Y2 () = (OPCoord) y + dy - 1;
	
 	if (box != this->bounding_box) {
 		this->view->UserChangedGeometry (this->bounding_box);
 	}
}

/*
 *	Return a window's frame size. This is style dependent
 *	and may be different for the left, right, top and
 *	bottom borders.
 */

void
OPGrailWindow::GetFrameSize (OPCoordOut xl, OPCoordOut yt,
							 OPCoordOut xr, OPCoordOut yb)
{
	OP_METHOD (GetFrameSize, 4);
	OP_ARG (1, OPCoordOut, xl);
	OP_ARG (2, OPCoordOut, yt);
	OP_ARG (3, OPCoordOut, xr);
	OP_ARG (4, OPCoordOut, yb);
	
	//  The Frame size doesn't matter. (Read below).
	
	xl = 0;
	yt = 0;
	xr = 0;
	yb = 0;
}



/*
 *	Return a window's frame size. This is style dependent
 *	and may be different for the left, right, top and
 *	bottom borders. This is a static member !
 */

void
OPGrailWindow::GetFrameSize (OPWindowStyle style,
							 OPCoordOut xl, OPCoordOut yt,
							 OPCoordOut xr, OPCoordOut yb)
{
	OP_MEMBER (GetFrameSize, 5);
	OP_ARG (1, OPWindowStyle, style);
	OP_ARG (2, OPCoordOut, xl);
	OP_ARG (3, OPCoordOut, yt);
	OP_ARG (4, OPCoordOut, xr);
	OP_ARG (5, OPCoordOut, yb);

	//  A X client can't ask the WM for the frame size without a window.
	//  So this static method can't be implemented.
	//  The rest of the implementation is done according to this limitation.
	
	xl = 0;
	yt = 0;
	xr = 0;
	yb = 0;
}



/*
 *	Start drawing into the window. This will first draw into
 *	the buffering pixmap.
 */

void
OPGrailWindow::BeginDraw (OPRectIn box, OPRelCoord margin)
{
	OP_METHOD (BeginDraw, 2);
	OP_ARG (1, OPRectIn, box);
	OP_ARG (2, OPRelCoord, margin);
	
	super::BeginDraw (box, margin);

	this->window->xredraw_rec.x      = (int)box.X1();
	this->window->xredraw_rec.y      = (int)box.Y1();
	this->window->xredraw_rec.width  = (int)box.DX();
	this->window->xredraw_rec.height = (int)box.DY();
}



/*
 *	Start drawing into the window. This will first draw into
 *	the buffering pixmap.
 */

void
OPGrailWindow::BeginDraw (OPCoord x1, OPCoord y1, OPCoord x2, OPCoord y2)
{
	OP_METHOD (BeginDraw, 4);
	OP_ARG (1, OPCoord, x1);
	OP_ARG (2, OPCoord, y1);
	OP_ARG (3, OPCoord, x2);
	OP_ARG (4, OPCoord, y2);
	super::BeginDraw (x1, y1, x2, y2);

	this->window->xredraw_rec.x      = (int)x1;
	this->window->xredraw_rec.y      = (int)y1;
	this->window->xredraw_rec.width  = (int)(x2-x1+1);
	this->window->xredraw_rec.height = (int)(y2-y1+1);
}



/*
 *	End of drawing. Invalidate the drawing rectangle in the
 *	window on screen.
 */

void
OPGrailWindow::EndDraw ()
{
	OP_METHOD (EndDraw, 0);
	super::EndDraw ();
	
	this->Flush ();
}


static void
DrawImage (GfxSurface* gfx, int x, int y, int dx, int dy,
		   Window xwindow)
{
#if 0
    XImage ximage = { 0 };

	ximage.width  = gfx->DX ();
	ximage.height = gfx->DY ();
	ximage.format = ZPixmap;
	ximage.data   = (char*) gfx->Line (0);
	
	ximage.byte_order       = LSBFirst;                  // endianness !!!
	ximage.bitmap_unit      = 32;
	ximage.bitmap_bit_order = MSBFirst;
	ximage.bitmap_pad       = 32;
	ximage.depth            = 32;
	ximage.bytes_per_line   = gfx->Pitch ();
	ximage.bits_per_pixel   = 32;
	ximage.red_mask         = 0x00FF0000;
	ximage.green_mask       = 0x0000FF00;
	ximage.blue_mask        = 0x000000FF;
	
	Drawable drawable = xwindow;
	GC gc = XCreateGC (xdpy, drawable, 0, 0);
	Status status = XInitImage (&ximage);
	XPutImage (xdpy, drawable, gc, &ximage, x, y, x, y, dx, dy);
	XFreeGC (xdpy, gc);
#else
	Drawable drawable = xwindow;
	GC gc = XCreateGC (xdpy, drawable, 0, 0);
	int y2 = y+dy;
	int x2 = x+dx;
	for (int iy = y; iy < y2; iy++) {
	    Card32 color = -1;
		Card32 old_pixel = 0;
	    for (int ix = x; ix < x2; ix++) {
			Card32 pixel = gfx->GetPixelRGB (ix, iy) & 0x00FFFFFF;
			if (old_pixel != pixel) {
			    Card8  red   = (pixel & 0x00FF0000) >> 16;
				Card8  green = (pixel & 0x0000FF00) >> 8;
				Card8  blue  = (pixel & 0x000000FF) >> 0;
				color  = 0;
				color |= (red << 8) & 0xF800;
				color |= (green << 3) & 0x07E0;
				color |= (blue >> 3) & 0x001F;
				old_pixel = pixel;
				XSetForeground (xdpy, gc, color);
			}
		    XDrawPoint (xdpy, drawable, gc, ix, iy);
	    }
	}
#endif
}


/*
 *	Send the graphic output to the screen. This will update
 *	the part of the window which has been marked as invalid
 *	using the BeginDraw/EndDraw methods.
 */

void
OPGrailWindow::Flush ()
{
	OP_METHOD (Flush, 0);
	
	DrawImage (this->surface,
			   this->window->xredraw_rec.x,
			   this->window->xredraw_rec.y,
			   this->window->xredraw_rec.width,
			   this->window->xredraw_rec.height,
			   this->window->xwindow);
	
	XFlush (xdpy);
}


/*
 *	Redraw the window.
 */

void
OPGrailWindow::Redraw ()
{
 	OP_METHOD (Redraw, 0);
	
	DrawImage (this->surface, 0, 0, (int)(this->dx), (int)(this->dy),
			   this->window->xwindow);
	
XFlush(xdpy);
}



/*
 *	Redraw the icon (miniaturized window).
 */

void
OPGrailWindow::RedrawIcon ()
{
	OP_METHOD (RedrawIcon, 0);
	
	//	NIY
}



/*
 *	Find which OPaC window -- if any -- corresponds to the specified
 *	coordinates. Look for the nth window (0 is the topmost).
 */


Bool
OPGrailWindow::FindOwnWindow (OPCoord x, OPCoord y, OPGrailWindowOut wdo,
															Count index)
{
	OP_MEMBER (FindOwnWindow, 4);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	OP_ARG (3, OPGrailWindowOut, wdo);
	OP_ARG (4, Count, index);

	unsigned long addr;
	unsigned char *inst;
	Atom xinst_atom;
	Atom xtype_atom;
	int format, sx, sy;
	unsigned long nitems;
	unsigned long bytes_after;

	Window root, parent;
	Window* children;
 	unsigned int nchildren;

	XWindowAttributes attributes;

	int rel_x, rel_y, trel_x, trel_y;
	Window child, temp;

	unsigned int width, height, xx, yy, border, depth;

	// Get the topmost window

	XTranslateCoordinates (xdpy,
						   DefaultRootWindow(xdpy),
						   DefaultRootWindow(xdpy),
						   x, y,
						   &rel_x, &rel_y,
						   &child);
	
	if(!child)
		return FALSE; // We're on root window...

	// List the direct children of Root

 	XQueryTree(xdpy,
						 DefaultRootWindow(xdpy),
						 &root, 
						 &parent,
						 &children,
						 &nchildren);

	// Traverse children list

 	for( int i = nchildren-1; i>=0; i--) {
		child=0;

		// Checks if this children is visible on screen

		XGetWindowAttributes(xdpy, children[i], &attributes);

		if( attributes.map_state == IsViewable ) {
		
			XTranslateCoordinates(xdpy,
														DefaultRootWindow(xdpy),
														children[i],
														x, y,
														&rel_x, &rel_y,
														&child);

			XGetGeometry(xdpy, children[i], &root, 
									 &xx, &yy, 
									 &width, &height, 
									 &border, &depth);

			// Check if this children contains x, y

			if( rel_x >= 0 && rel_x <=width &&
					rel_y >= 0 && rel_y <= height) { // This is a candidate

				// Check if this is the stack level we want

				if( index == 0 ) {
					temp = parent = children[i];

					while( child ) { // Descent to the inner window
						temp = child;
						XTranslateCoordinates(xdpy,
																	parent,
																	temp,
																	rel_x, rel_y,
																	&trel_x, &trel_y,
																	&child);

						rel_x = trel_x; rel_y=trel_y;
						parent=temp;
					}

					// Now temp is the inner window, is it an OPaC window ?
					xinst_atom = XInternAtom(xdpy, OP_XPROP_WIN_INST, False);

					XGetWindowProperty(xdpy, temp,
														 xinst_atom,
														 0, 8,
														 0,
														 XA_STRING,
														 &xtype_atom,
														 &format,
														 &nitems,
														 &bytes_after,
														 (unsigned char**)&inst);

					if(xtype_atom != None) { // It is an OPaC Window !!
            sscanf(inst, "%x", &addr);
						wdo = (OPGrailWindow*)addr;
            
            XFree(inst);
						return TRUE; // Bye, we won !!
					} else {
            XFree(inst);
						return FALSE; // Too bad
          }
				} else // The desired window is below, let's go down
					index--;
			}
		}
	}

	// No window found, too bad

	return FALSE;
				
}


Bool
OPGrailWindow::FindOPaCWindow (OPCoord x, OPCoord y, OPaC_RemoteAppDesc* desc,
							   Count index)
{
	OP_MEMBER (FindOPaCWindow, 4);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	OP_ARG (3, OPaC_RemoteAppDesc*, desc);
	OP_ARG (4, Count, index);
	
	if (desc == 0) {
		return FALSE;
	}
	
  // NIY
	
	return TRUE;
}
