/*
 *	colorwell.cxx
 *
 *	Color well class definition.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/colorwell.h>
#include <OPaC/dragview.h>
#include <OPaC/event.h>
#include <OPaC/grail.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>

#include <OPaC/pubmacro.h>
#include <OPaC/notmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPColorWell);

/****************************************************************/

OP_PUBLISH_SLOT (OPColorWell, color, ReturnColor, SetColor, "Color", OPColorPixel);
OP_PUBLISH_SLOT (OPColorWell, red, ReturnRed, SetRed, "Red component", Real);
OP_PUBLISH_SLOT (OPColorWell, green, ReturnGreen, SetGreen, "Greeen component", Real);
OP_PUBLISH_SLOT (OPColorWell, blue, ReturnBlue, SetBlue, "Blue component", Real);
OP_PUBLISH_SLOT (OPColorWell, transparency, ReturnTransparency, SetTransparency, "Transparency", Real);
OP_PUBLISH_SLOT (OPColorWell, luminance, ReturnLuminance, SetLuminance, "Luminance", Real);

OP_NOTIFY_CHANGE (OPColorWell, color, Changed);
OP_NOTIFY_CHANGE (OPColorWell, red, Changed);
OP_NOTIFY_CHANGE (OPColorWell, green, Changed);
OP_NOTIFY_CHANGE (OPColorWell, blue, Changed);
OP_NOTIFY_CHANGE (OPColorWell, transparency, Changed);
OP_NOTIFY_CHANGE (OPColorWell, luminance, Changed);

/****************************************************************/

OPColorWell::OPColorWell ()
{
	this->color  = OP_COLOR_NONE;
	this->shape |= OP_CTRL_RELIEF;
	this->shape |= OP_CTRL_ABOVE;
	this->frame  = 4.0;
}

OPColorWell::~OPColorWell ()
{
}

void
OPColorWell::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	static OPStorageSync info ("<OPColorWell color frame>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << this->color << this->frame;
	storage << OPStorageSync::end;
}

void
OPColorWell::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> this->color >> this->frame;
	storage >> OPStorageSync::end;
}

Bool
OPColorWell::SetColor (OPColorPixelIn color)
{
	OP_METHOD (SetColor, 1);
	OP_ARG (1, OPColorPixelIn, color);
	
	if (color != this->color) {
		this->color = color;
		this->QuickReportChange ();
	}
	
	return TRUE;
}

OPColorPixel
OPColorWell::ReturnColor () const
{
	OP_METHOD (ReturnColor, 0);
	return this->color;
}

Bool
OPColorWell::SetRed (Real x)
{
	OP_METHOD (SetRed, 1);
	OP_ARG (1, Real, x);
	
	OPColorPixel color (x /* this->color.RealRed () */,
						this->color.RealGreen (),
						this->color.RealBlue (),
						this->color.RealTransparency ());
	
	this->SetColor (color);
	
	return TRUE;
}

Bool
OPColorWell::SetGreen (Real x)
{
	OP_METHOD (SetGreen, 1);
	OP_ARG (1, Real, x);
	
	OPColorPixel color (this->color.RealRed (),
						x /*this->color.RealGreen ()*/,
						this->color.RealBlue (),
						this->color.RealTransparency ());
	
	this->SetColor (color);
	
	return TRUE;
}

Bool
OPColorWell::SetBlue (Real x)
{
	OP_METHOD (SetBlue, 1);
	OP_ARG (1, Real, x);
	
	OPColorPixel color (this->color.RealRed (),
						this->color.RealGreen (),
						x /*this->color.RealBlue ()*/,
						this->color.RealTransparency ());
	
	this->SetColor (color);
	
	return TRUE;
}

Bool
OPColorWell::SetTransparency (Real x)
{
	OP_METHOD (SetTransparency, 1);
	OP_ARG (1, Real, x);
	
	OPColorPixel color (this->color.RealRed (),
						this->color.RealGreen (),
						this->color.RealBlue (),
						x /*this->color.RealTransparency ()*/);
	
	this->SetColor (color);
	
	return TRUE;
}

Bool
OPColorWell::SetLuminance (Real x)
{
	OP_METHOD (SetLuminance, 1);
	OP_ARG (1, Real, x);
	
	OPColorPixel color (this->color.RealRed () * x,
						this->color.RealGreen () * x,
						this->color.RealBlue () * x,
						this->color.RealTransparency ());
	
	this->SetColor (color);
	
	return TRUE;
}

Real
OPColorWell::ReturnRed () const
{
	OP_METHOD (ReturnRed, 0);
	return this->color.RealRed ();
}

Real
OPColorWell::ReturnGreen () const
{
	OP_METHOD (ReturnGreen, 0);
	return this->color.RealGreen ();
}

Real
OPColorWell::ReturnBlue () const
{
	OP_METHOD (ReturnBlue, 0);
	return this->color.RealBlue ();
}

Real
OPColorWell::ReturnTransparency () const
{
	OP_METHOD (ReturnTransparency, 0);
	return this->color.RealTransparency ();
}

Real
OPColorWell::ReturnLuminance () const
{
	OP_METHOD (ReturnLuminance, 0);
	return this->color.RealLuminance ();
}

void
OPColorWell::Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what)
{
	OP_METHOD (Draw, 3);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	OP_ARG (3, OPDrawOp, what);
	
	super::Draw (port, rect, what);
	
	if (what & OP_DRAW_OBJECT) {
		
		const OPAdornerRef& adorner = port->ReturnAdorner ();
		const OPLookManRef& lookman = OPLookMan::Current ();
		
		OPCoord margin = (this->shape & OP_CTRL_NOFRAME) ? 0 : lookman->ReturnWidgetFrameSize ();
		OPCoord effect = (this->shape & OP_CTRL_RELIEF) ? lookman->Return3DEffectWidth () : 0;
		
		margin += this->frame;
		margin += effect;
		
		port->SetForeColor (OP_COLOR_WHITE);
		port->FillRect (this->box, margin);

		port->SetForeColor (this->color);
		port->FillRect (this->box, margin);
		
		port->SetForeColor (OP_COLOR_BLACK);
		port->DrawRect (this->box, margin);
	}
}



/*
 *	Handle color dropping.
 */

Bool
OPColorWell::DragAndDrop (OPRectIn box, OPObjectIn initiator, OPDragDropProtocol ddp)
{
	OP_METHOD (DragAndDrop, 3);
	OP_ARG (1, OPRectIn, box);
	OP_ARG (2, OPObjectIn, initiator);
	OP_ARG (3, OPDragDropProtocol, ddp);
	
	if (ddp == OP_DDP_DRAGnDROP_CHECK) {
		
		OPObjectRef object;
		initiator->QueryData ("drag_object", object);
		
		if ( (object.IsValid ())
		  && (object->IsKindOf (OP_RUNTIME (OPValue))) ) {
			OPValueRef value = object;
			if (value->ReturnType () == OP_TYPE_COLOR) {
				return TRUE;
			}
		}
		
		return FALSE;
	}
	
	if (ddp == OP_DDP_DRAGnDROP_COMMIT) {
		
		super::DragAndDrop (box, initiator, OP_DDP_DRAGnDROP_CANCEL);
		
		OPObjectRef object;
		initiator->QueryData ("drag_object", object);
		
		if ( (object.IsValid ())
		  && (object->IsKindOf (OP_RUNTIME (OPValue))) ) {
			
			OPValueRef value = object;
			if (value->ReturnType () == OP_TYPE_COLOR) {
				this->SetColor (value->ReturnColor ());
				this->Display ();
				return TRUE;
			}
		}
		
		return FALSE;
	}
	
	return super::DragAndDrop (box, initiator, ddp);
}



void
OPColorWell::ButtonPress (OPEventIn event)
{
	OP_METHOD (ButtonPress, 1);
	OP_ARG (1, OPEventIn, event);
	
	if (event->ReturnChange () == OP_EV_BUTTON_LEFT) {
		
		OPCoord x = event->ReturnX ();
		OPCoord y = event->ReturnY ();
		
		OPDragViewRef  drag  = OP_NEW (OPDragView);
		OPColorWellRef well  = OP_NEW (OPColorWell);
		OPValueRef     color = this->color;
		
		const OPLookManRef& lookman = OPLookMan::Current ();
		
		OPCoord cx = (this->box.X1 () + this->box.X2 ()) / 2;
		OPCoord cy = (this->box.Y1 () + this->box.Y2 ()) / 2;
		
		well->SetParent (this);
		well->SetOrigin (cx-8, cy-8);
		well->SetSize (17, 17);
		well->SetFlagOkDetect (TRUE);
		well->SetColor (this->color);
		well->frame  = 0.0;
		well->shape |= OP_CTRL_NOFRAME;
		well->shape &= ~OP_CTRL_RELIEF;
		
		drag->SetAutoStop (TRUE);
		drag->SetAutoOops (FALSE);
		drag->Setup (well, TRUE);
		drag->SetHotSpot (x-cx+8, y-cy+8);
		drag->SetDragObject (color);
		
		OPEvent::HandOverToWindow (drag);
	}
	
	super::ButtonPress (event);
}
