/*
 *	event.cxx
 *
 *	Platform independent part of the event handler.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/new.h>
#include <OPaC/event.h>
#include <OPaC/grail.h>
#include <OPaC/window.h>
#include <OPaC/app.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPEvent);

/****************************************************************/

/*
 *	Initialise an event record. In fact, there should be
 *	just one such record in any OPaC application.
 */

OPEvent::OPEvent ()
{
	OP_METHOD (OPEvent, 0);
	
	this->type    = OP_EVENT_NONE;
	this->id      = 0;
	
	this->window  = 0;
	
	this->x       = 0;
	this->y       = 0;
	this->buttons = 0;
	
	this->change  = 0;
	this->data    = 0;
	this->name    = "";
	this->code    = 0;
	
	this->key.type      = 0;
	this->key.value     = 0;
	this->key.code      = 0;
	this->key.modifiers = 0;
	
	this->is_dead = FALSE;
}



/*
 *	Destroy the event : this will free the references to
 *	the associated window, data and key name.
 */

OPEvent::~OPEvent ()
{
	OP_METHOD (~OPEvent, 0);
	
	this->window = 0;
	this->data   = 0;
	this->name   = "";
}



/*
 *	Store an event.
 */

void
OPEvent::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	static OPStorageSync info ("<OPEvent type id window x y buttons change data name code"
							   " <key type code value modifiers> >");
	storage << info;
	storage << OPStorageSync::begin;
	storage << this->type << this->id;
	storage << this->window << this->x << this->y << this->buttons;
	storage << this->change << this->data << this->name << this->code;
	storage << OPStorageSync::begin;
	storage << this->key.type << this->key.code << this->key.value << this->key.modifiers;
	storage << OPStorageSync::end;
	storage << OPStorageSync::end;
}


/*
 *	Restore an event.
 */

void
OPEvent::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> this->type >> this->id;
	storage >> this->window >> this->x >> this->y >> this->buttons;
	storage >> this->change >> this->data >> this->name >> this->code;
	storage >> OPStorageSync::begin;
	storage >> this->key.type >> this->key.code >> this->key.value >> this->key.modifiers;
	storage >> OPStorageSync::end;
	storage >> OPStorageSync::end;
}

/****************************************************************/

/*
 *	Dispatch the event to the window and let it do the job
 *	for us.
 */

void
OPEvent::DispatchEvent ()
{
	OP_METHOD (DispatchEvent, 0);
	
	if (this->window.IsValid ()) {
		this->window->DispatchEvent (OPEvent::current);
	}
}

