/*
 *	except.cxx
 *
 *	This file implements the C++ base exception class and the
 *	associated methods.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/except.h>
#include <OPaC/new.h>

#include <string.h>

OP_IMPLEMENTATION_FILE;

/****************************************************************/

#define	MAX_EXC_SIZE	500

/*
 *	Allocate memory for an exception. This will always use
 *	the same static buffer, since only one exception at a
 *	time can be tolerated.
 */

void*
OPException::operator new (size_t size)
{
	OP_OPERATOR (new, 1);
	OP_ARG (1, size_t, size);
	OP_ASSERT (size <= MAX_EXC_SIZE);
	
	static char exc_buffer[MAX_EXC_SIZE];
	return exc_buffer;
}

OPException::OPException ()
{
	this->throw_frame = OPaC_CallFrame::ReturnCallFrame () [0];
}

/*
 *	Return the source line number where the throw occurred.
 */

Count
OPException::ReturnThrowSrcLine () const
{
	OP_MEMBER (ReturnThrowSrcLine, 0);
	return this->throw_frame.ReturnFileLine ();
}



/*
 *	Return the source file name where the throw occurred.
 */

const LatinChar*
OPException::ReturnThrowSrcFile () const
{
	OP_MEMBER (ReturnThrowSrcFile, 0);
	return this->throw_frame.ReturnFileName ();
}

/****************************************************************/
