/*
 *	fontman.cxx
 *
 *	GRAphical Abstraction Layer Font Manager. It is responsible
 *	for the initialisation of the font data base.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/grail.h>
#include <OPaC/database.h>
#include <OPaC/new.h>

/****************************************************************/

#define	MAX_FONT_NAME	100

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPFontMan);

/****************************************************************/

OPFontManRef OPFontMan::current = 0;

/****************************************************************/

struct FontFamilyDB {
	OPDataBaseRef		db;
	OPEntityModelRef	model;
	OPEntityRef			entity;
};

/****************************************************************/

#if 0
	if (type == TRUETYPE_FONTTYPE) {
		
		if (def->entity.IsValid () == FALSE) {
			
			//	First pass : just remember about the family names !
			
			OPEntityRef data = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPEntity);
			OPStringRef name = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPString);
			OPListRef   list = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPList);
			
			name = font_name;
			
			data->Initialise (def->model);
			data->SetAttribute (1, name);
			data->SetAttribute (2, list);
			
			def->db->Insert (data);
			
		} else {
			
			//	Second pass : record the different available font styles.
			
			OPObjectRef object;
			def->entity->GetAttribute (2, object);
			
			OPListRef   list = object;
			OPStringRef name = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPString);
			
			name = font_style;
			
			list->Insert (name);
		}
	}
	
	return 1;
}
#endif

/****************************************************************/

/*
 *	Initialise the font manager. This will build the data
 *	base...
 */

OPFontMan::OPFontMan ()
{
	OP_METHOD (OPFontMan, 0);
	
	this->font_families  = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPDataBase);
	this->font_fam_model = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPEntityModel);
	this->font_def_model = OP_NEW_FROM_ZONE (OPaC_sys_zone, OPEntityModel);
	
	this->font_fam_model->Initialise (this->font_families, "FontFamilyModel", 2);
	this->font_fam_model->AddAttribute ("Name");
	this->font_fam_model->AddAttribute ("StyleList");
	
	this->font_def_model->Initialise (this->font_families, "FontDefinitionModel", 3);
	this->font_def_model->AddAttribute ("FamilyName");
	this->font_def_model->AddAttribute ("StyleName");
	this->font_def_model->AddAttribute ("Weight");
	
	FontFamilyDB def;
	
	def.db     = this->font_families;
	def.model  = this->font_fam_model;
	def.entity = 0;
	
	//	First, get a list of every available font family. This can be done
	//	using a Windows Iterator Callback function. The enumerated fonts
	//	aren't sorted... It's up to the DB object to do the work.
	
	/*****/
	
	//	Now, we have access to a sorted list of font families.
	
	Count     num   = this->font_families->TotalCount ();
	Count     index = 0;
	OPAtomRef sort  = "Name";
	
	while (index < num) {
		
		OPObjectRef object;
		OPStringRef string;
		LatinChar   name[MAX_FONT_NAME];
		Count       len = 0;
		
		//	Search the entity described by the index.
		
		this->font_families->GetEntity (sort, index++, def.entity);
		OP_ASSERT (def.entity.IsValid ());
		
		//	Get the font family name from the entity.
		
		def.entity->GetAttribute (1, object);
		OP_ASSERT (object.IsValid ());
		string     = object;
		
		//	Extract the string so that it can be used by the Win16 call-back
		//	function. We would have been pleased to be able to use OPString
		//	directly, but we can't...
		
		len = string->CountCharacters ();
		
		if (len >= MAX_FONT_NAME)
			continue;
		
		string->GetStringCopy (name, len); name[len] = 0;
		
		//	Enumerate the fonts within the named family.
		
		/*****/
	}
}



/*
 *	Free the associated data base.
 */

OPFontMan::~OPFontMan ()
{
	OP_METHOD (~OPFontMan, 0);
	
	if (this->font_families.IsValid ()) {
		this->font_families->RemoveAll ();
	}
}



/*
 *	Save the font manager. This is really silly ! We won't
 *	do anything.
 */

void
OPFontMan::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	static OPStorageSync info ("<OPFontMan>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << OPStorageSync::end;
}



/*
 *	Restore the font manager. This is really silly ! We
 *	won't do anything.
 */

void
OPFontMan::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> OPStorageSync::end;
}



/*
 *	Return the current font families data base.
 */

void
OPFontMan::GetFontFamiliesDB (OPDataBaseOut db) const
{
	OP_METHOD (GetFontFamiliesDB, 1);
	OP_ARG (1, OPDataBaseOut, db);
	db = this->font_families;
}



/*
 *	Initialise the system font manager.
 */

void
OPFontMan::InitSystemFontMan (OPFontManIn font_man)
{
	OP_MEMBER (InitSystemFontMan, 1);
	OP_ARG (1, OPFontManIn, font_man);
	OP_ASSERT (OPFontMan::current.IsValid () == FALSE);
	OPFontMan::current = font_man;
}


void
OPFontMan::KillSystemFontMan ()
{
	OP_MEMBER (KillSystemFontMan, 0);
	OPFontMan::current = 0;
}

