/*
 *	targexport.cxx
 *
 *	Implements a Target Export class which registers information about
 *	the methods.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/new.h>
#include <OPaC/targetmacro.h>

OP_IMPLEMENTATION_FILE;

/****************************************************************/

OPaC_TargetExport*  OPaC_TargetExport::head  = 0;
OPaC_TargetExport*  OPaC_TargetExport::tail  = 0;

/****************************************************************/

OPaC_TargetExport::OPaC_TargetExport (OPaC_TrampoFuncM func, OPRuntimeIn rt)
{
	this->next = 0;
	this->prev = this->tail;
	
	(this->prev ? this->prev->next : this->head) = this;
	(this->next ? this->next->prev : this->tail) = this;

	this->info.runtime   = & rt;
	this->info.method    = 0;
	this->info.m_trampo  = func;
	this->info.o_trampo  = 0;
	this->info.a_trampo  = 0;
	this->info.num_param = 0;
	
	OPMessageRef message;
	OPObject*    object  = 0;
	
	(func)(message, object, this);
}

OPaC_TargetExport::OPaC_TargetExport (OPaC_TrampoFuncA func,
									  const char* name, OPRuntimeIn rt)
{
	this->next = 0;
	this->prev = this->tail;
	
	(this->prev ? this->prev->next : this->head) = this;
	(this->next ? this->next->prev : this->tail) = this;

	this->info.runtime   = & rt;
	this->info.method    = name;
	this->info.m_trampo  = 0;
	this->info.o_trampo  = 0;
	this->info.a_trampo  = func;
	this->info.num_param = 0;
}

OPaC_TargetExport::OPaC_TargetExport (OPaC_TrampoFuncO func,
									  const char* name, OPRuntimeIn rt)
{
	this->next = 0;
	this->prev = this->tail;
	
	(this->prev ? this->prev->next : this->head) = this;
	(this->next ? this->next->prev : this->tail) = this;

	this->info.runtime   = & rt;
	this->info.method    = name;
	this->info.m_trampo  = 0;
	this->info.o_trampo  = func;
	this->info.a_trampo  = 0;
	this->info.num_param = 0;
}
	
void
OPaC_TargetExport::SetArg (const LatinChar* type, const LatinChar* arg, const LatinChar* desc)
{
	OP_ASSERT (this->info.num_param < 10);
	
	this->info.param[this->info.num_param].type = type;
	this->info.param[this->info.num_param].name = arg;
	this->info.param[this->info.num_param].desc = desc;
	this->info.num_param++;
}

void
OPaC_TargetExport::SetCall (const char* call)
{
	this->info.method = call;
}
