/*
 *	buildrect.cxx
 *
 *	Helper class used to edit and define rectangles.
 *	
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "buildrect.h"

#include <OPaC/view.h>

#include <OPaC/notmacro.h>
#include <OPaC/pubmacro.h>
#include <OPaC/targetmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPBuildRect);

OP_TARGET_ACTOR (OPBuildRect, Changed);

OP_NOTIFY_EVENT (OPBuildRect, RectChanged);

OP_NOTIFY_CHANGE (OPBuildRect, rectangle, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, x1, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, y1, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, x2, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, y2, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, dx, RectChanged);
OP_NOTIFY_CHANGE (OPBuildRect, dy, RectChanged);

OP_PUBLISH_SLOT (OPBuildRect, rectangle, ReturnRect, SetRect, "Rectangle", OPRect);
OP_PUBLISH_SLOT (OPBuildRect, x1, ReturnX1, SetX1, "X1", OPCoord);
OP_PUBLISH_SLOT (OPBuildRect, y1, ReturnY1, SetY1, "Y1", OPCoord);
OP_PUBLISH_SLOT (OPBuildRect, x2, ReturnX2, SetX2, "X2", OPCoord);
OP_PUBLISH_SLOT (OPBuildRect, y2, ReturnY2, SetY2, "Y2", OPCoord);
OP_PUBLISH_SLOT (OPBuildRect, dx, ReturnDX, SetDX, "DX", OPCoord);
OP_PUBLISH_SLOT (OPBuildRect, dy, ReturnDY, SetDY, "DY", OPCoord);

/****************************************************************/

OPBuildRect::OPBuildRect ()
{
	OP_METHOD (OPBuildRect, 0);
}

OPBuildRect::~OPBuildRect ()
{
	OP_METHOD (~OPBuildRect, 0);
}

Bool
OPBuildRect::Changed ()
{
	OP_METHOD (Changed, 0);
	
	this->Report (notify_RectChanged);
	
	if (this->view.IsValid ()) {
		OPViewRef parent = this->view->ReturnParent ();
		if (parent.IsValid ()) {
			parent->Display ();
		}
	}
	
	return TRUE;
}

/****************************************************************/

Bool
OPBuildRect::SetDX (OPCoord dx)
{
	OPCoord dy = this->ReturnDY ();
	this->rect.SetSize (dx, dy);
	return this->Changed ();
}

Bool
OPBuildRect::SetDY (OPCoord dy)
{
	OPCoord dx = this->ReturnDX ();
	this->rect.SetSize (dx, dy);
	return this->Changed ();
}
