/*
 *	buildrect.h
 *
 *	Helper class used to edit and define rectangles.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _BUILDRECT_H_
#define _BUILDRECT_H_

#include <OPaC/object.h>
#include <OPaC/opexport.h>
#include <OPaC/rect.h>

/****************************************************************/

APP_EXPORT_CLASS (OPBuildRect);

/****************************************************************/

APP_DEFINE_REF (OPBuildRect, OPObject);
APP_DEFINE_ARRAY (OPBuildRect, OPObject);

/****************************************************************/

class OPBuildRect : public OPObject
{
private:
	OPRect				rect;
	OPViewRef			view;
	
protected:
	OPBuildRect ();
	virtual ~ OPBuildRect ();
	
public:
	virtual Bool Changed ();
	
	Bool SetView (OPViewIn view) { this->view = view; return TRUE; }
	Bool SetRect (OPRectIn rect) { this->rect = rect; return this->Changed (); }
	
	Bool SetX1 (OPCoord x1) { OPCoord y1 = this->rect.Y1 (); this->rect.SetOrigin (x1, y1); return this->Changed (); }
	Bool SetY1 (OPCoord y1) { OPCoord x1 = this->rect.X1 (); this->rect.SetOrigin (x1, y1); return this->Changed (); }
	Bool SetX2 (OPCoord x2) { this->rect.X2 () = x2; return this->Changed (); }
	Bool SetY2 (OPCoord y2) { this->rect.Y2 () = y2; return this->Changed (); }
	Bool SetDX (OPCoord dx);
	Bool SetDY (OPCoord dy);
	
	const OPRect& ReturnRect () const { return this->rect; }
	
	OPCoord ReturnX1 () const { return this->rect.X1 (); }
	OPCoord ReturnY1 () const { return this->rect.Y1 (); }
	OPCoord ReturnX2 () const { return this->rect.X2 (); }
	OPCoord ReturnY2 () const { return this->rect.Y2 (); }
	OPCoord ReturnDX () const { return this->rect.X2 () - this->rect.X1 () + 1; }
	OPCoord ReturnDY () const { return this->rect.Y2 () - this->rect.Y1 () + 1; }

	APP_DEFINE_CLASS (OPBuildRect, OPObject);
};

/****************************************************************/

#endif /* _BUILDRECT_H_ */
