/*
 *	contargetdlg.cxx
 *
 *	Fill in a view with the information needed to establish the
 *	connections between an event and an action.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "buildedit.h"
#include "connectdlg.h"

#include <OPaC/button.h>
#include <OPaC/database.h>
#include <OPaC/new.h>
#include <OPaC/selgrid.h>
#include <OPaC/string.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPConActorDialog);

/****************************************************************/

void
OPConActorDialog::CreateSourceList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy)
{
	this->source_list = OP_NEW (OPDBScrollList);
	
	this->source_list->SetParent (this);
	this->source_list->SetOrigin (x, y);
	this->source_list->SetSize (dx, dy);
	this->source_list->SetupDataBase ("Source events");
	this->source_list->Update ();
	
	OPBuildEdit::MakeEventList (this->source_list, this->source);
}

void
OPConActorDialog::CreateTargetList (OPCoord x, OPCoord y, OPCoord dx, OPCoord dy)
{
	this->target_list = OP_NEW (OPDBScrollList);
	
	this->target_list->SetParent (this);
	this->target_list->SetOrigin (x, y);
	this->target_list->SetSize (dx, dy);
	this->target_list->SetupDataBase ("Target actions");
	this->target_list->Update ();
	
	OPBuildEdit::MakeTargetList (this->target_list, this->target);
}

/****************************************************************/

OPConActorDialog::OPConActorDialog ()
{
	this->SetAttribute ("OPaC.volatile.observer");
}

OPConActorDialog::~OPConActorDialog ()
{
}

/****************************************************************/

/*
 *	This call-back method is invoked when the user presses
 *	on the "Connect" button. It will establish a connection
 *	(or several, if possible).
 */

void
OPConActorDialog::Connect ()
{
	OP_METHOD (Connect, 0);
	
	Count nth = 0;
	
	if (this->source_list->CountSelectedItems ()) {
		while (nth < this->target_list->CountSelectedItems ()) {
			OPAtomRef src;
			OPAtomRef dst;
			this->source_list->GetSelectedName (src);
			this->target_list->GetSelectedName (dst, nth++);
			this->source->AttachObserver (this->target, src, dst);
		}
	}
}

