/*
 *	conviewer.h
 *
 *	The connection viewer is a tool which allows to inspect and
 *	re-order connections between events and actors.
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _CONVIEWER_H_
#define _CONVIEWER_H_

#include <OPaC/dbscrlist.h>
#include <OPaC/opexport.h>
#include <OPaC/view.h>

/****************************************************************/

APP_EXPORT_CLASS (OPBuildConViewer);

/****************************************************************/

APP_DEFINE_REF (OPBuildConViewer, OPView);
APP_DEFINE_ARRAY (OPBuildConViewer, OPView);

/****************************************************************/

class OPBuildConViewer : public OPView
{

private:
	OPDBScrollListRef	view_events;			//	view showing a list of events
	OPDBScrollListRef	view_actors;			//	view showing the corresponding actors
	
	OPButtonRef			button_disconnect;		//	button used to disconnect an actor
	Count				source_selected;		//	selected source
	
	OPAtomRef			event_name;
	OPObjectRef			observer;
	OPAtomRef			observer_name;
	Card32				observer_id;
	
	OPObjectRef			object;					//	object being inspected
	OPViewRef			attached_events;
	
protected:
	OPBuildConViewer ();
	virtual ~ OPBuildConViewer ();
	
	virtual Bool UpdateSelectedInfo ();
	
public:
	virtual void Initialise (OPObjectIn object, OPStringIn title_events, OPStringIn title_actors);
	
	virtual Bool GetSelectedEvent (OPAtomOut event);
	virtual Bool GetSelectedActor (OPObjectOut observer, OPAtomOut name, Card32Out id);
	
	virtual Bool Update ();
	virtual Bool EventCellActivated ();
	virtual Bool ActorCellActivated ();
	virtual Bool ActorCellDragged ();
	virtual Bool DisconnectActor ();
	
	APP_DEFINE_CLASS (OPBuildConViewer, OPView);
};

/****************************************************************/

#endif /* _CONVIEWER_H_ */
