/*
 *	icobjbrowser.cxx
 *
 *	This class aggregates the Object Browser class with icons and
 *	scroller.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "icobjbrowser.h"

#include <OPaC/array.h>
#include <OPaC/button.h>
#include <OPaC/database.h>
#include <OPaC/datarep.h>
#include <OPaC/dragpal.h>
#include <OPaC/icon.h>
#include <OPaC/new.h>
#include <OPaC/phantom.h>
#include <OPaC/scroller.h>
#include <OPaC/selgrid.h>
#include <OPaC/string.h>

#include <OPaC/targetmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPIcobjBrowser);

OP_TARGET_OBSERVER	(OPIcobjBrowser, IconsChanged);

/****************************************************************/

OPIcobjBrowser::OPIcobjBrowser ()
{
	OP_METHOD (OPIcobjBrowser, 0);
	this->SetAttribute ("OPaC.volatile.observer");
}

OPIcobjBrowser::~OPIcobjBrowser ()
{
	OP_METHOD (~OPIcobjBrowser, 0);
}

void
OPIcobjBrowser::Initialise (OPObjectIn root, Count panes)
{
	OP_METHOD (Initialise, 2);
	OP_ARG (1, OPObjectIn, root);
	OP_ARG (2, Count, panes);
	
	this->root  = root;
	this->panes = panes;
	
	OPCoord x  = 0;
	OPCoord y  = 0;
	OPCoord dx = 0;
	OPCoord dy = 0;
	
	this->GetOrigin (x, y);
	this->GetSize (dx, dy);
	
	dx /= this->panes;
	
	this->dragpal = OP_NEW (OPArray);
	this->dragpal->SetSize (this->panes);
	
	for (Count i = 0; i < this->panes; i++) {
		this->dragpal[i] = OP_NEW (OPDragPalette);
	}
	
	this->provider = OP_NEW (OPObjRootProvider);
	this->provider->Initialise (root);
	
	this->browser = OP_NEW (OPObjectBrowser);
	this->browser->SetParent (this);
	this->browser->SetOrigin (x, y+50);
	this->browser->SetSize (dx*this->panes, dy-50-20);
	this->browser->Initialise (this->panes, this->provider, "OPSelGrid");
	this->browser->AttachObserver (this, "IconsChanged");
	
	this->scroller = OP_NEW (OPScroller);
	
	OPDataReportRef datarep1 = OP_NEW (OPDataReport);
	OPDataReportRef datarep2 = OP_NEW (OPDataReport);
	OPDataReportRef datarep3 = OP_NEW (OPDataReport);
	
	this->scroller->SetParent (this);
	this->scroller->Initialise (x, y+dy-20+3, dx*this->panes, TRUE);
	
	datarep1->Initialise (this->browser, "VisibleValueChanged", "visible_value", this->scroller, "visible_value");
	datarep2->Initialise (this->browser, "VisibleRatioChanged", "visible_ratio", this->scroller, "visible_ratio");
	datarep3->Initialise (this->scroller, "Changed", "visible_value", this->browser, "visible_value");
}



void
OPIcobjBrowser::IconsChanged (OPObjectIn obj, Card32 id)
{
	OP_METHOD (IconsChanged, 2);
	OP_ARG (1, OPObjectIn, obj);
	OP_ARG (2, Card32, id);
	
	for (Count i = 0; i < this->panes; i++) {
		
		OPViewRef child;
		
		this->dragpal[i]->GetFirstChild (child);
		this->dragpal[i]->SetParent (OPViewNull);
		
		if (child.IsValid ()) {
			child->SetParent (OPViewNull);
		}
	}
	
	Count index = this->browser->ReturnZeroPane ();
	OPIconRef   icon;
	OPObjectRef object;
	OPObjectPaneRef pane;
	OPGenericProviderRef provider;
	OPSelGridRef grid;
	OPCoord     x, y;
	this->GetOrigin (x, y);
	
	i = 0;
	
	while ( (this->browser->GetIcon (index, icon, x))
		 && (this->browser->GetPane (index, pane))
		 && (this->browser->GetProvider (index, provider))
		 && (pane->GetCellGrid (grid)) ) {
		
		Count col, line;
		
		if (grid->FindActiveHilite (col, line)) {
			if (provider->GetObject (line, object)) {
				
				if (this->slot.IsValid ()) {
					
					OPObjectRef phantom;
					static OPAtomRef atom_phantom_object = "OPaC.phantom.object";
					
					if (object->GetAttribute (atom_phantom_object, phantom) == FALSE) {
						
						OPStringRef path;
						OPObjectRef root = this->slot;
						Count       pos  = 0;
						
						this->browser->GetSelectedPath (path);
						
						if (path->Find ('.', pos)) {
							OPStringRef rel_path;
							path->MakeMiddle (pos+1, 1000, rel_path);
							path = rel_path;
							root = this->root;
						}
						
						phantom = OP_NEW (OPPhantom);
						OPPhantom* ptr = (OPPhantom*)(phantom.ReturnRawObject ());
						
						ptr->Initialise (root, path, FALSE);
						
						object->SetAttribute (atom_phantom_object, phantom);
					}
				}
				
				icon->RemoveAttribute ("embedded_object_for_drag");
				icon->RemoveAttribute ("embedded_object_for_connect");
				icon->SetAttribute ("embedded_object_for_drag", object);
				icon->SetAttribute ("embedded_object_for_connect", object);
				icon->SetParent (this);
				icon->SetOrigin (x, y);
				
				this->dragpal[i]->Initialise (icon);
				this->dragpal[i]->SetAutoCopy (FALSE);
			}
		}
		
		i++;
		index++;
	}
	
	this->Display ();
}


Bool
OPIcobjBrowser::SetSlot (OPDataSlotIn slot)
{
	OP_METHOD (SetSlot, 1);
	OP_ARG (1, OPDataSlotIn, slot);
	
	this->slot = slot;
	return TRUE;
}

Bool
OPIcobjBrowser::GetSlot (OPDataSlotOut slot) const
{
	OP_METHOD (GetSlot, 1);
	OP_ARG (1, OPDataSlotOut, slot);
	
	slot = this->slot;
	return slot.IsValid ();
}
