/*
 *	springview.h
 *
 *	The spring view class draws the overlays representing a band's
 *	springs.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_IB_SPRINGVIEW_H_
#define	_IB_SPRINGVIEW_H_

#ifndef	_OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

APP_EXPORT_CLASS (OPSpringView);
APP_DEFINE_REF (OPSpringView, OPView);
APP_DEFINE_ARRAY (OPSpringView, OPView);

/****************************************************************/

class OPSpringView : public OPView
{
protected:
	OPSpringBandRef			band;					//	associated spring band
	OPWindowRef				window;					//	window registered to
	OPViewRef				hilite;					//	hilite this view or all views
	
protected:
	OPSpringView ();
	virtual ~ OPSpringView ();
	
	virtual void Disconnect ();
	
public:
	virtual void Delete ();							/* OVERRIDDEN */
	
	virtual Bool SetHiliteView (OPViewIn view);
	virtual Bool SetHiliteAll ();
	
	virtual Bool SetSpringBand (OPSpringBandIn band);
	virtual const OPSpringBandRef& ReturnSpringBand () const;
	
	virtual void DrawOverlay (OPGrailPortIn port, OPRectIn rect);
	
	APP_DEFINE_CLASS (OPSpringView, OPView);
};

/****************************************************************/

#endif /* _IB_SPRINGVIEW_H_ */
